@@ -1,6 +1,6 @@
 /* ============================================================
-* QupZilla - WebKit based browser
-* Copyright (C) 2010-2016  David Rosca <nowrep@gmail.com>
+* QupZilla - Qt web browser
+* Copyright (C) 2010-2017 David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
 #include "lineedit.h"
 #include "history.h"
 #include "locationbar.h"
-#include "searchtoolbar.h"
 #include "websearchbar.h"
 #include "pluginproxy.h"
 #include "sidebar.h"
@@ -55,7 +54,6 @@
 #include "webtab.h"
 #include "speeddial.h"
 #include "menubar.h"
-#include "qtwin.h"
 #include "bookmarkstools.h"
 #include "bookmarksmenu.h"
 #include "historymenu.h"
@@ -81,6 +79,15 @@
 #include <QToolTip>
 #include <QScrollArea>
 #include <QCollator>
+#include <QPrintDialog>
+#include <QPrinter>
+#include <QTemporaryFile>
+
+#ifdef Q_OS_WIN
+#include <qt_windows.h>
+#else
+#include "fileprinter.h"
+#endif
 
 #ifdef QZ_WS_X11
 #include <QX11Info>
@@ -182,17 +189,10 @@ void BrowserWindow::postLaunch()
         else if (mApp->afterLaunch() == MainApplication::RestoreSession && mApp->restoreManager()) {
             addTab = !mApp->restoreSession(this, mApp->restoreManager()->restoreData());
         }
-        else {
-            // Restore pinned tabs also when not restoring session
-            m_tabWidget->restorePinnedTabs();
-        }
         break;
 
-    case Qz::BW_MacFirstWindow:
-        m_tabWidget->restorePinnedTabs();
-        // fallthrough
-
     case Qz::BW_NewWindow:
+    case Qz::BW_MacFirstWindow:
         addTab = true;
         break;
 
@@ -241,9 +241,8 @@ void BrowserWindow::postLaunch()
     QTimer::singleShot(0, this, [this]() {
         // Scroll to current tab
         tabWidget()->tabBar()->ensureVisible();
-
         // Update focus
-        if (LocationBar::convertUrlToText(weView()->page()->requestedUrl()).isEmpty())
+        if (!m_startPage && LocationBar::convertUrlToText(weView()->page()->requestedUrl()).isEmpty())
             locationBar()->setFocus();
         else
             weView()->setFocus();
@@ -614,6 +613,52 @@ void BrowserWindow::changeEncoding()
     }
 }
 
+void BrowserWindow::printPage()
+{
+    QPrintDialog* dialog = new QPrintDialog(this);
+    dialog->setOptions(QAbstractPrintDialog::PrintToFile | QAbstractPrintDialog::PrintShowPageSize);
+#ifndef Q_OS_WIN
+    dialog->setOption(QAbstractPrintDialog::PrintPageRange);
+    if (Qz::FilePrinter::cupsAvailable()) {
+        dialog->setOption(QAbstractPrintDialog::PrintCollateCopies);
+    }
+#endif
+    dialog->printer()->setCreator(tr("QupZilla %1 (%2)").arg(Qz::VERSION, Qz::WWWADDRESS));
+    dialog->printer()->setDocName(QzTools::getFileNameFromUrl(weView()->url()));
+
+    if (dialog->exec() == QDialog::Accepted) {
+        if (dialog->printer()->outputFormat() == QPrinter::PdfFormat) {
+            weView()->page()->printToPdf(dialog->printer()->outputFileName(), dialog->printer()->pageLayout());
+            delete dialog;
+        } else {
+            weView()->page()->printToPdf([=](const QByteArray &data) {
+                if (!data.isEmpty()) {
+                    QTemporaryFile tempFile(QDir::tempPath() + QSL("/QupZillaPrintXXXXXX.pdf"));
+                    tempFile.setAutoRemove(false);
+                    if (tempFile.open()) {
+                        qint64 bytesWritten = tempFile.write(data);
+                        tempFile.close();
+                        if (bytesWritten == data.size()) {
+#ifdef Q_OS_WIN
+                            QString printerName = '"' + dialog->printer()->printerName() + '"';
+                            // This may bring up a PDF viewer window, and even keep it open, but it is the best we can do without adding third-party dependencies.
+                            // lpr is not installed by default on Windows, and it also can only print PDF if the printer handles it in hardware.
+                            ShellExecuteW((HWND)winId(), L"printto", (LPCWSTR)tempFile.fileName().utf16(), (LPCWSTR)printerName.utf16(), NULL, SW_HIDE);
+                            // TODO: When can we delete the file?
+#else
+                            Qz::FilePrinter::printFile(dialog->printer(), tempFile.fileName(), Qz::FilePrinter::SystemDeletesFiles, Qz::FilePrinter::SystemSelectsPages);
+#endif
+                        } else {
+                            tempFile.remove();
+                        }
+                    }
+                }
+                delete dialog;
+            }, dialog->printer()->pageLayout());
+        }
+    }
+}
+
 void BrowserWindow::bookmarkPage()
 {
     TabbedWebView* view = weView();
@@ -754,6 +799,7 @@ void BrowserWindow::toggleShowBookmarksToolbar()
     setUpdatesEnabled(true);
 
     Settings().setValue("Browser-View-Settings/showBookmarksToolbar", m_bookmarksToolbar->isVisible());
+    Settings().setValue("Browser-View-Settings/instantBookmarksToolbar", false);
 }
 
 void BrowserWindow::toggleShowNavigationToolbar()
@@ -829,11 +875,6 @@ void BrowserWindow::currentTabChanged()
     m_ipLabel->setText(view->getIp());
     view->setFocus();
 
-    SearchToolBar* search = searchToolBar();
-    if (search) {
-        search->setWebView(view);
-    }
-
     updateLoadingActions();
 
     // Setting correct tab order (LocationBar -> WebSearchBar -> WebView)
@@ -897,7 +938,7 @@ void BrowserWindow::createToolbarsMenu(QMenu* menu)
 
     action = menu->addAction(tr("&Bookmarks Toolbar"), this, SLOT(toggleShowBookmarksToolbar()));
     action->setCheckable(true);
-    action->setChecked(m_bookmarksToolbar->isVisible());
+    action->setChecked(Settings().value("Browser-View-Settings/showBookmarksToolbar").toBool());
 
     menu->addSeparator();
 
@@ -984,16 +1025,9 @@ void BrowserWindow::webSearch()
 
 void BrowserWindow::searchOnPage()
 {
-    SearchToolBar* toolBar = searchToolBar();
-
-    if (!toolBar) {
-        const int searchPos = 2;
-
-        toolBar = new SearchToolBar(weView(), this);
-        m_mainLayout->insertWidget(searchPos, toolBar);
+    if (weView() && weView()->webTab()) {
+        weView()->webTab()->showSearchToolBar();
     }
-
-    toolBar->focusSearchLine();
 }
 
 void BrowserWindow::openFile()
@@ -1075,12 +1109,10 @@ bool BrowserWindow::event(QEvent* event)
             statusBar()->hide();
 
             m_navigationContainer->hide();
-            m_navigationToolbar->buttonExitFullscreen()->setVisible(true);
+            m_navigationToolbar->buttonExitFullscreen()->show();
         }
         else if (ev->oldState() & Qt::WindowFullScreen && !(windowState() & Qt::WindowFullScreen)) {
             // Leave fullscreen
-            setWindowState(m_windowStates);
-
             statusBar()->setVisible(m_statusBarVisible);
 #ifndef Q_OS_MAC
             menuBar()->setVisible(m_menuBarVisible);
@@ -1088,8 +1120,10 @@ bool BrowserWindow::event(QEvent* event)
 
             m_navigationContainer->show();
             m_navigationToolbar->setSuperMenuVisible(!m_menuBarVisible);
-            m_navigationToolbar->buttonExitFullscreen()->setVisible(false);
+            m_navigationToolbar->buttonExitFullscreen()->hide();
             m_isHtmlFullScreen = false;
+
+            setWindowState(m_windowStates);
         }
 
         if (m_hideNavigationTimer) {
@@ -1361,18 +1395,6 @@ void BrowserWindow::closeEvent(QCloseEvent* event)
     event->accept();
 }
 
-SearchToolBar* BrowserWindow::searchToolBar() const
-{
-    SearchToolBar* toolBar = 0;
-    const int searchPos = 2;
-
-    if (m_mainLayout->count() == searchPos + 1) {
-        toolBar = qobject_cast<SearchToolBar*>(m_mainLayout->itemAt(searchPos)->widget());
-    }
-
-    return toolBar;
-}
-
 void BrowserWindow::closeWindow()
 {
 #ifdef Q_OS_MAC
