@@ -1,6 +1,6 @@
 /* ============================================================
-* QupZilla - WebKit based browser
-* Copyright (C) 2010-2016 David Rosca <nowrep@gmail.com>
+* QupZilla - Qt web browser
+* Copyright (C) 2010-2017 David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -36,7 +36,6 @@ class MainMenu;
 class TabWidget;
 class TabbedWebView;
 class LineEdit;
-class SearchToolBar;
 class HistoryMenu;
 class BookmarksMenu;
 class BookmarksToolbar;
@@ -155,6 +154,7 @@ private slots:
     void webSearch();
     void searchOnPage();
     void changeEncoding();
+    void printPage();
 
     void saveSettings();
     void hideNavigationSlot();
@@ -166,8 +166,6 @@ private:
     void keyReleaseEvent(QKeyEvent* event);
     void closeEvent(QCloseEvent* event);
 
-    SearchToolBar* searchToolBar() const;
-
     void setupUi();
     void setupMenu();
 
