@@ -75,9 +75,6 @@ public:
     bool restoreState(const QVector<WebTab::SavedTab> &tabs, int currentTab);
     void closeRecoveryTab();
 
-    void savePinnedTabs();
-    void restorePinnedTabs();
-
     void setCurrentIndex(int index);
 
     void nextTab();
@@ -117,6 +114,8 @@ public slots:
     void reloadAllTabs();
     void stopTab(int index);
     void closeAllButCurrent(int index);
+    void closeToRight(int index);
+    void closeToLeft(int index);
     void detachTab(int index);
     void restoreClosedTab(QObject* obj = 0);
     void restoreAllClosedTabs();
