@@ -81,6 +81,15 @@ QUrl QzTools::pixmapToDataUrl(const QPixmap &pix)
     return data.isEmpty() ? QUrl() : QUrl(QSL("data:image/png;base64,") + data);
 }
 
+QPixmap QzTools::dpiAwarePixmap(const QString &path)
+{
+    const QIcon icon(path);
+    if (icon.availableSizes().isEmpty()) {
+        return QPixmap(path);
+    }
+    return icon.pixmap(icon.availableSizes().at(0));
+}
+
 QString QzTools::readAllFileContents(const QString &filename)
 {
     return QString::fromUtf8(readAllFileByteContents(filename));
@@ -198,13 +207,13 @@ QString QzTools::fromPunycode(const QString &str)
     return decoded.left(decoded.size() - 4);
 }
 
-QString QzTools::escapeSqlString(QString urlString)
+QString QzTools::escapeSqlGlobString(QString urlString)
 {
-    const static QString &escapeString = QL1S("!");
-    urlString.replace(escapeString, escapeString + escapeString);
-    urlString.replace(QL1S("_"), escapeString + QL1S("_"));
-    urlString.replace(QL1S("%"), escapeString + QL1S("%"));
-
+    urlString.replace(QL1C('['), QStringLiteral("[["));
+    urlString.replace(QL1C(']'), QStringLiteral("[]]"));
+    urlString.replace(QStringLiteral("[["), QStringLiteral("[[]"));
+    urlString.replace(QL1C('*'), QStringLiteral("[*]"));
+    urlString.replace(QL1C('?'), QStringLiteral("[?]"));
     return urlString;
 }
 
@@ -352,27 +361,32 @@ QPixmap QzTools::createPixmapForSite(const QIcon &icon, const QString &title, co
     const QFontMetrics fontMetrics = QApplication::fontMetrics();
     const int padding = 4;
     const int maxWidth = fontMetrics.width(title.length() > url.length() ? title : url) + 3 * padding + 16;
-
     const int width = qMin(maxWidth, 150);
     const int height = fontMetrics.height() * 2 + fontMetrics.leading() + 2 * padding;
 
-    QPixmap pixmap(width, height);
+    QPixmap pixmap(width * qApp->devicePixelRatio(), height * qApp->devicePixelRatio());
+    pixmap.setDevicePixelRatio(qApp->devicePixelRatio());
+
     QPainter painter(&pixmap);
+    painter.setRenderHint(QPainter::Antialiasing);
 
     // Draw background
     QPen pen(Qt::black);
     pen.setWidth(1);
     painter.setPen(pen);
 
-    painter.fillRect(QRect(0, 0, width, height), Qt::white);
-    painter.drawRect(0, 0, width - 1, height - 1);
+    QPainterPath path;
+    path.addRect(QRect(0, 0, width, height));
+
+    painter.fillPath(path, Qt::white);
+    painter.drawPath(path);
 
     // Draw icon
-    QRect iconRect(0, 0, 16 + 2 * padding, height);
+    QRect iconRect(padding, 0, 16, height);
     icon.paint(&painter, iconRect);
 
     // Draw title
-    QRect titleRect(iconRect.width(), padding, width - padding - iconRect.width(), fontMetrics.height());
+    QRect titleRect(iconRect.right() + padding, padding, width - padding - iconRect.right(), fontMetrics.height());
     painter.drawText(titleRect, fontMetrics.elidedText(title, Qt::ElideRight, titleRect.width()));
 
     // Draw url
